package Algebra;

import Brueche.Bruch;

public class Koerper {

  protected int n; //Koerpergrad
  protected int Diskriminante;
  protected long[] f = new long[n]; //erzeugendes Polynom
  protected double[] NS = new double[n]; //Nullstellen von f
  protected Bruch[][] ZBasis = new Bruch[n][n];

  //Initialisierung:

  public Koerper(int n) {
    this.n = n;
  }

  public Koerper(int n, int Diskriminante) {
    this.n = n;
    this.Diskriminante = Diskriminante;
  }

  public void initialf(long[] f) {
    this.f = f;
  }

  public void initialNS(double[] NS) {
    this.NS = NS;
  }

  public void initialZBasis(Bruch[][] ZBasis) {
    this.ZBasis = ZBasis;
  }

  //Rueckgabe der entsprechenden Objektkomponenten:

  public int n() {
    return n;
  }

  public int Diskriminante() {
    return Diskriminante;
  }

  public long[] f() {
    return f;
  }

  public double[] NS() {
    return NS;
  }

  public Bruch[][] ZBasis() {
    return ZBasis;
  }

  //sonstige Methoden:

  public double phiVon(int i, double[] Koeff) {
    double phiVon = 0.0;
    for(byte j=0; j<n; j++)
      phiVon += Koeff[j] * Rechnen.hoch(NS[i],j);
    return phiVon;
  }

  public Bruch[] rhoumwandeln(long[] x) {
    Bruch[] xBruch = new Bruch[n];
    for (byte h=0; h<n; h++)
      xBruch[h] = new Bruch(0,1);
    for (byte i=0; i<n; i++)
      for (byte j=0; j<n; j++)
        xBruch[j] = Bruch.add(xBruch[j], Bruch.gemischtmult(x[i], ZBasis[i][j]));
    return xBruch;
  }
    

  public static Bruch[] rhoadd(Bruch[] x, Bruch[] y) {
    return Rechnen.BruchPolynomAdd(x, y);
  }

  public static Bruch[] rhomult(Bruch[] x, Bruch[] y) {
    return Rechnen.BruchPolynomMult(x, y);
  }

  public Bruch[] rhoreduzieren(Bruch[] x) {
    Bruch[] Rest = x;
    Bruch[] fBruch = new Bruch[n+1];
    for (byte h=0; h<n; h++)
      fBruch[h] = new Bruch(f[h], 1);
    fBruch[n] = new Bruch(1,1);
    for (int i=x.length-1; i>=n; i--) {
      Bruch Faktor = new Bruch(-Rest[i].Zaehler(), Rest[i].Nenner());
      for (int j=0; j<=n; j++)
        Rest[i-j] = Bruch.add(Rest[i-j], Bruch.mult(Faktor, fBruch[n-j]));
    }
    Bruch[] Ergebnis = new Bruch[n];
    for (byte j=0; j<Rechnen.min(n, x.length); j++)
      Ergebnis[j] = Rest[j];
    for (int j=x.length; j<n; j++)
      Ergebnis[j] = new Bruch(0,1);
    return Ergebnis;
  }

  public boolean rhogleich(Bruch[] x, Bruch[] y) {
    Bruch[] xreduziert = rhoreduzieren(x);
    Bruch[] yreduziert = rhoreduzieren(y);
    byte i=0;
    while (i<n && Bruch.gleich(xreduziert[i], yreduziert[i]))
      i++;
    return i==n;
  }
}
